program Richardson_4p0

!  Analisador e estimador de erros de discretizao
!     com base na extrapolao de Richardson

!  Verso original 1.0 ( 1 Fev 04)
!  Verso          2.0 (10 Jan 07)
!  Verso          1.1 (30 Abr 07)
!  Verso          3.0 (10 Nov 07)
!  Verso          3.1 ( 2 Dez 07)
!  Verso          3.2 (10 Jul 09)
!  Verso          4.0 (18 Fev 13)

!  Autor: Carlos Henrique Marchi
!         DEMEC/TC/UFPR, Curitiba, PR

! Adaptao para verso 4.0: Mrcio Andr Martins

!  MODELO NUMRICO (resumo)

!     Faz extrapolaes simples e mltiplas dos resultados numricos
!     Calcula ordem aparente de resultados extrapolados ou no
!     Calcula erro estimado de resultados numricos com e sem extrapolaes,
!        com estimadores Richardson, GCI, convergente e multicoeficientes
!     Razo de refino de malha (r) constante ou varivel;
!        mtodo da bisseo para obter ordens aparente com r varivel
!     Malhas uniformes
!     Preciso qudrupla
!     Linguagem Fortran 2003
!     Tipo de projeto: Console Application
!     Aplicativo usado: Fortran Intel 11.0
!     Os dados devem conter no mnimo trs malhas
!     Os valores de pL e dpL so necessrios; seus valores podem ser reais
!     Resultados em branco de parmetros significa que no h dados suficientes
!        para calcul-lo ou no  possvel calcul-lo

!  Adaptao: obtm resultados com os estimadores GCI (Roache) e multicoeficiente para soluo sem extrapolao,
!             estimador convergente para soluo numrica convergente, e uma proposta de estimador para soluo 
!             obtida com o emprego de MER

! ARQUIVOS envolvidos no programa:

!    Fontes (prefixo Richardson_4p0_):
!       data_hora.f90  = obtm a data e hora do sistema
!       variaveis.f90  = define todas as variveis globais do programa
!       dados.f90      = l e escreve os dados do programa
!       calculos.f90   = calcula as variveis
!       resultados.f90 = escreve resultados e apresenta grficos
!       main.f90       = programa principal
      
!    Dados:
!       Richardson_4p0.in   = leitura do nome do arquivo principal de dados do CASO a analisar
!       "arq_dados"         = leitura dos dados principais para o caso de um arquivo por malha
!       "arq_dados_unico"   = leitura dos dados principais para o caso de um arquivo para todas as malha
!       "arq_numerico(nm)"  = nomes dos nm arquivos das solues numricas (um arquivo para cada
!                             malha mas com os resultados de todas as variveis de interesse )
!       "arq_numerico(1)"   = nome do arquivo que contm as solues numricas (um nico arquivo para todas
!                             as malhas, com os resultados de todas as variveis de interesse )

!    Sada principal:
!       "caso"             = sada principal do programa

!    Aplicativos auxiliares:
!       Notepad.exe        = editor de arquivos
!       Wgnuplot.exe       = programa gerador de grfico

!    Arquivos de sada auxiliares para grficos:
!       p0.dat      = dados para grfico de p0        (ordem assinttica)
!       p1.dat      = dados para grfico de pV(2)     (segunda ordem verdadeira) 
!       p2.dat      = dados para grfico de pV(3)     (terceira ordem verdadeira) 
!       pU_h.dat    = dados para grfico de pU_h      (ordem aparente equivalente para soluo extrapolada) 
!       pU_i.dat    = dados para grfico de pU_i      (ordem aparente equivalente para soluo extrapolada)
!       pU_bi.dat   = dados para grfico de pU_bi     (ordem aparente equivalente para soluo biextrapolada)
!       pU_c.dat    =   dados para grfico de pU_c    (ordem aparente equivalente para soluo convergente)
!       pU_mer.dat  =   dados para grfico de pU_MER  (ordem aparente equivalente para soluo com MER)

!       U_12.dat   = dados para grfico de UGCI  (estimador GCI (Roache) para Th)
!       U_13.dat   = dados para grfico de Umc   (estimador multicoeficiente com base em MER para Th)
!       U_14.dat   = dados para grfico de Uc    (estimador convergente para soluo convergente)
!       U_15.dat   = dados para grfico de Um    (estimador Um para Tm (soluo com MER))
!       
!    Arquivos de comandos para Wgnuplot (prefixo Richardson_4p0_):
!       pU.gnu    = grfico de ordens
!       U.gnu     = grfico de erros estimados (U*) 
!       
!    Arquivo texto com informaes bsicas sobre o programa:
!       LEIA-ME_Richardson_4p0.txt

!-------------------------------------------------------------------------------

use resultados

!-------------------------------------------------------------------------------

implicit none

!-------------------------------------------------------------------------------

  call le_dados

  open(out,file=trim(adjustl(caso) ) )

  call data_e_hora ( dia, hora )

  head = trim(title)//" "//trim(dia)//"'"

  call escreve_dados

!-------------------------------------------------------------------------------

  call calcula

  call mostra_resultados

  close (out)

  note_caso = 'notepad ' // trim(adjustl(caso))

  dos = system(note_caso) ! lista arquivo de resultados

end

