module variaveis

! Objetivo: definir as variveis globais do programa

use portlib
use data_hora

!-------------------------------------------------------------------------------

implicit none

real*16,parameter :: great  = 1.0q+11
real*16,parameter :: limite = 1.0q+10

! *** Variveis principais ***

integer :: entrada  ! tipo de arquivo de entrada
                    ! 0 --> uma malha por arquivo
                    ! 1 --> todas as malhas em um nico arquivo

integer :: variavel ! nmero da varivel de interesse nos arquivos de dados

integer :: nm  ! nmero de malhas

integer :: i   ! ndice das malhas
               ! i = 1  ---> malha mais grossa
               ! i = nm ---> malha mais fina

integer :: m   ! nmero de extrapolaes
               ! m = 1    ---> uma extrapolao
               ! m = nm-1 ---> nmero mximo de extrapolaes

real*16 :: r21 ! razo de h entre as malhas grossa(2) e fina(1)
real*16 :: r32 ! razo de h entre as malhas supergrossa(3) e grossa(2)

real*16 :: pL  ! ordem assinttica do erro verdadeiro
real*16 :: dpL ! variao entre as ordens verdadeiras do erro verdadeiro

integer :: tipo_refino ! tipo de refino de malha (r): 1=constante; 2=varivel
real*16 :: p  ! varivel auxiliar


! dados necessrios apenas para tipo_refino = 2 (r varivel)
! para obter a ordem aparente (pU) atravs do mtodo da bisseo
integer :: numero_iteracoes ! nmero mximo de iteraes
real*16 :: tol              ! tolerncia
real*16 :: pU_min           ! limite inferior de pU
real*16 :: pU_max           ! limite superior de pU

! *** Variveis de campo (vetores) com dimenso (1:nm) ***

character*60,dimension(:),allocatable :: arq_numerico ! nomes dos arquivos das solues numricas

real*16,dimension(:),allocatable :: h       ! comprimento de cada elemento da malha

real*16,dimension(:),allocatable   :: Th    ! soluo numrica em cada malha
real*16,dimension(:,:),allocatable :: Th1   ! soluo numrica em cada malha (auxiliar para o caso de arquivo nico com todas as malhas)

real*16,dimension(:),allocatable     :: pV  ! ordens verdadeiras do erro verdadeiro
character*5,dimension(:),allocatable :: pVt ! pV em formato texto

real*16,dimension(:),allocatable :: Ti_12         ! Th extrapolado com as malhas 1 e 2 e pL
real*16,dimension(:),allocatable :: Ti_13         ! Th extrapolado com as malhas 1 e 3 e pL
real*16,dimension(:),allocatable :: Ti_23         ! Th extrapolado com as malhas 2 e 3 e pL
real*16,dimension(:),allocatable :: Ti_pU         ! Th extrapolado com as malhas 1 a 3 e pU_h
real*16,dimension(:),allocatable :: Ti_pU1        ! Th extrapolado com as malhas 1 a 3 e pU_h equivalente
real*16,dimension(:),allocatable :: Tc_12         ! Mdia de Ti_12 com Ti_pU
real*16,dimension(:),allocatable :: Tc_13         ! Mdia de Ti_13 com Ti_pU
real*16,dimension(:),allocatable :: Tbi_pU        ! Th biextrapolado com malhas 1 a 5 e pU_i
real*16,dimension(:),allocatable :: Tbi_pU1       ! Th biextrapolado com malhas 1 a 5 e pU_i equivalente
real*16,dimension(:),allocatable :: T_MER         ! Th multiextrapolado (ltimo nvel de extrapolao em cada malha)
real*16,dimension(:),allocatable :: pU_MER        ! Ordem aparente de Th multiextrapolado (ltimo nvel de extrapolao em cada malha) 
real*16,dimension(:),allocatable :: U_Ri_mpsi_MER ! Estimador Richardson com psi mdio de MER internveis (psi atribuido  malha intermediria do trio)
real*16,dimension(:),allocatable :: U_Ri_p0       ! Estimadores Richardson com base na ordem assinttica (p0)
real*16,dimension(:),allocatable :: U_Ri_pU       ! Estimadores Richardson com base na ordem aparente (pU)
real*16,dimension(:),allocatable :: T_inf_p0      ! Soluo extrapolada com p0
real*16,dimension(:),allocatable :: T_inf_pU      ! Soluo extrapolada com pU
real*16,dimension(:),allocatable :: T_c           ! Soluo numrica convergente
real*16,dimension(:),allocatable :: U_c           ! Estimador convergente
real*16,dimension(:),allocatable :: U_GCI         ! Estimador GCI (Roache)
real*16,dimension(:),allocatable :: U_mc          ! Estimador multicoeficiente, com base em MER
real*16,dimension(:),allocatable :: psi_inter     ! Vetor razo de convergncia inter nveis para Tm
real*16,dimension(:),allocatable :: psi_v         ! Vetor razo de convergncia psi pata Th
real*16,dimension(:),allocatable :: psi_c         ! Vetor razo de convergncia psi da soluo convergente
real*16,dimension(:),allocatable :: psi_i1        ! Vetor razo de convergncia psi de Th extrapolado com as malhas 1 a 3 e pU_h
real*16,dimension(:),allocatable :: psi_bi1       ! Vetor razo de convergncia psi de Th biextrapolado com malhas 1 a 5 e pU_i

! Estimador de Richardson (Uri)
real*16,dimension(:),allocatable :: Uri_12  ! erro estimado de Th com as malhas 1 e 2 e pL
real*16,dimension(:),allocatable :: Uri_13  ! erro estimado de Th com as malhas 1 e 3 e pL
real*16,dimension(:),allocatable :: Uri_23  ! erro estimado de Th com as malhas 2 e 3 e pL
real*16,dimension(:),allocatable :: Uri_pU  ! erro estimado de Th com as malhas 1 a 3 e pU_h
real*16,dimension(:),allocatable :: Uri_p   ! erro estimado de Th com as malhas 1 a 3, pU_h e p

! Estimador Convergente (Uc)
real*16,dimension(:),allocatable :: Uc_12   ! erro estimado de Tc_12
real*16,dimension(:),allocatable :: Uc_13   ! erro estimado de Tc_13

real*16,dimension(:),allocatable :: pU_h    ! ordem aparente do erro estimado de Th
real*16,dimension(:),allocatable :: pU_h1   ! ordem aparente equivalente do erro estimado de Th
real*16,dimension(:),allocatable :: pU_i    ! ordem aparente do erro estimado de Ti_pU
real*16,dimension(:),allocatable :: pU_i1   ! ordem aparente equivalente do erro estimado de Ti_pU
real*16,dimension(:),allocatable :: pU_c12  ! ordem aparente do erro estimado de Tc_12
real*16,dimension(:),allocatable :: pU_c13  ! ordem aparente do erro estimado de Tc_13
real*16,dimension(:),allocatable :: pU_bi   ! ordem aparente do erro estimado de Tbi_pU
real*16,dimension(:),allocatable :: pU_bi1  ! ordem aparente equivalente do erro estimado de Tbi_pU
real*16,dimension(:),allocatable :: pU_c1   ! ordem aparente equivalente do erro estimado para soluo convergente

integer,dimension(:),allocatable:: it_pU_h   ! nmero de iteraes para obter pU_h
integer,dimension(:),allocatable:: it_pU_i   ! nmero de iteraes para obter pU_i
integer,dimension(:),allocatable:: it_pU_c12 ! nmero de iteraes para obter pU_c12
integer,dimension(:),allocatable:: it_pU_c13 ! nmero de iteraes para obter pU_c13
integer,dimension(:),allocatable:: it_pU_bi  ! nmero de iteraes para obter pU_bi

! *** Variveis de campo (matrizes) com dimenso (1:nm,1:nm-1) ***

real*16,dimension(:,:),allocatable :: Um_12 ! erro estimado de Tm_12 com as malhas 1 e 2 e pV
real*16,dimension(:,:),allocatable :: Tm_12 ! Th multiextrapolado com as malhas 1 e 2 e pV

! *** vriaveis auxiliares ***

integer, parameter :: in   =  7 ! nmero do arquivo de entrada (dados)
integer, parameter :: out  = 10 ! nmero do arquivo de sada principal
integer, parameter :: out2 = 11 ! nmero do arquivo de sada secundrio

character*60  :: caso           ! nome do arquivo de sada
character*100  :: title          ! ttulo do grfico
character*202 :: head           ! ttulo do grfico + dia
character*12  :: dia            ! data da simulao
character*8   :: hora           ! horrio da simulao
character*90  :: note_caso      ! notepad + caso

character*80 :: arq_exato       ! nome do arquivo da soluo exata
character*80 :: arq_dados       ! nome do arquivo de dados principal para o caso de vrios arquivos, um para cada malha
character*80 :: arq_dados_unico ! nome do arquivo de dados principal para o caso de um unico arquivo, contendo dados de todas as malhas

integer :: dos                  ! acessa prompt dos

integer :: j, k ! auxiliares

!-------------------------------------------------------------------------------

end module variaveis


